const { isString } = require('lodash');
let _ = require('lodash');
let streetTypes = require('./street-types');
let unitTypes = require('./unit-types');

// regex patterns to help
let poBoxPattern = /^(g?po\s*box)\s*(\d*).*/i;
let lotPattern = /^(lot)\s*(\d*)\s*(.*)/i;
let unitPatternRegex = /^(unit|u|house|hse|flat|f|suite|townhouse|shed|site)\s*([a-zA-Z]{0,1}\d*)\s*(.*)/i;
let unitWithSlashPatternRegex = /^(unit|u|house|hse|flat|f|suite|townhouse|shed|site){0,1}\s*([a-zA-Z]{0,1}\d*)\s*\/{1,1}\s*(.*)/i;

const streetNameWithBlankStreetType = [
    "the boulevarde",
    "the boulevard",
    "the peninsula",
    "the bulwark",
    "the avenue",
    "kensington mews",
    "the ridgeway",
    "the strand",
    "the lanes",
    "the mall",
    "the terrace",
    "kingsway",
    "the grange",
    "the hill",
    "the grove",
    "broadway",
    "the parade",
    "the esplanade",
    "esplanade",
    "the promenade",
    "the circle",
    "the glades",
    "the ridge",
    "the scarp",
    "roney macs twist",
    "the broadway",
    "the lee",
    "the bounty",
    "the panorama",
    "castle court",
    "james cook island",
    "bells line of road",
    "the sanctuary",
    "the links",
    "the broadwaters",
    "south gateway",
    "the galley",
    "the crescent",
    "the promontory",
    "the anchorage",
    "the serpentine"
];

let _helper = (function() {

    let _helper = function() {

    };

    _helper.prototype.getStructured = function(addr, cb) {
        let obj = {};
        addr = addr.replace(/\"/g, '');
        addr = addr.replace(/[,]/g, ' ');

        let isPoBox = false;
        let matches = addr.match(poBoxPattern);
        if (matches) {
            isPoBox = true;
        }

        if (isPoBox) {
            obj = parsePoBoxAddress(obj, addr, matches);
        } else {
            obj = parseAddress(obj, addr, matches);
        }

        return obj;
    };

    function parsePoBoxAddress(obj, addr, matches) {
        obj['postalDeliveryType'] = poBoxType(matches[1]);
        obj['postalDeliveryNumberValue'] = matches[2];

        addr = addr.replace(matches[1], "");
        addr = addr.replace(matches[2], "");
        addr = addr.trim();

        let addressParts = addr.split(/[\s\t]+/g);

        if (addressParts.length < 3) {
            throw 'not enough parts to the address: ' + addr;
        }

        for (let b = 0; b < addressParts.length; b++) {
            s = addressParts[b];
            if (s.match(/^\d{4,4}$/)) {
                obj["postcode"] = s;
            } else if (stateFromString(s)) {
                obj["stateOrTerritory"] = s;
            } else {
                obj["suburbOrPlaceOrLocality"] = s;
            }
        }

        // final massaging
        if (addressParts.length == 4 && obj["postcode"]) {
            obj["suburbOrPlaceOrLocality"] = addressParts[0] + " " + addressParts[1];
        }

        if (addressParts.length == 3 && obj["postcode"] == null) {
            obj["suburbOrPlaceOrLocality"] = addressParts[0] + " " + addressParts[1];
        }

        if (obj["stateOrTerritory"] == null && obj["postcode"]) {
            obj["stateOrTerritory"] = stateFromPostcode(obj["postcode"]);
        }

        return obj;
    }

    function parseAddress(obj, address, matches) {
        let addr = address;
        matches = addr.match(lotPattern);
        if (matches) {
            obj["lotNumber"] = matches[2];
            addr = matches[3];
        }

        
        matches = addr.match(unitWithSlashPatternRegex);
        if (matches) {
            if (matches[1] && matches[2]) {
                obj['flatOrUnitType'] = unitTypes(matches[1]);
                obj["flatOrUnitNumber"] = matches[2];
            } else if (matches[2]) {
                obj['flatOrUnitType'] = 'U';
                obj["flatOrUnitNumber"] = matches[2];
            }
            
            addr = matches[3];
        } else {
            matches = addr.match(unitPatternRegex);
             if (matches) {
                obj['flatOrUnitType'] = unitTypes(matches[1]);
                obj["flatOrUnitNumber"] = matches[2];
                addr = matches[3];
            }
        }

        let streetNum = null;
        let streetName = null;
        let streetType = null;
        let suburb = null;
        let postcode = null;
        let state = null;
        let nextIndex = 0;

        let addressParts = addr.split(/[\s\t]+/g);
 
        if (addressParts.length < 5) {
            throw 'not enough parts to the address: ' + address;
        }

        streetNum = addressParts[0];
        matches = streetNum.match(/^(\d+)([A-Za-z]*)/);
        if (matches) {
            obj["houseNumber"] = matches[1];
            if (matches[2]) {
                obj["houseNumberSuffix"] = matches[2];
            }
        } else {
            throw 'invalid street number: ' + address;
        }

        if (streetNameWithBlankStreetType.indexOf(_.toLower(addressParts[1])) >= 0) {
            streetName = addressParts[1];
            streetType = '';
            nextIndex = 2;
        } else if (streetNameWithBlankStreetType.indexOf(_.toLower(addressParts[1] + ' ' + addressParts[2])) >= 0) {
            streetName = addressParts[1] + ' ' + addressParts[2];
            streetType = '';
            nextIndex = 3;
        } else if (streetNameWithBlankStreetType.indexOf(_.toLower(addressParts[1] + ' ' + addressParts[2] + ' ' + addressParts[3])) >= 0) {
            streetName = addressParts[1] + ' ' + addressParts[2] + ' ' + addressParts[3];
            streetType = '';
            nextIndex = 4;
        } else if (streetNameWithBlankStreetType.indexOf(_.toLower(addressParts[1] + ' ' + addressParts[2] + ' ' + addressParts[3] + ' ' + addressParts[4])) >= 0) {
            streetName = addressParts[1] + ' ' + addressParts[2] + ' ' + addressParts[3] + ' ' + addressParts[4];
            streetType = '';
            nextIndex = 5;
        } else { 
            for (let i=2; i < addressParts.length; i++) {
                streetType = streetTypes(addressParts[i])
                if (streetType) {
                    let streetNameParts = [];
                    for (let j=1; j < i; j++) {
                        streetNameParts.push(addressParts[j]);
                    }
                    streetName = streetNameParts.join(' ');
                    nextIndex = i+1;
                    break;
                }
            }
        }
        
        if (streetName && isString(streetName)) {
            obj["streetName"] = streetName.toUpperCase();
            if (streetType) {
                obj["streetType"] = streetType;
            }
        } else {
            throw 'invalid street name: ' + address;
        }

        let suburbParts = [];
        for (let b = nextIndex; b < addressParts.length; b++) {
            let s = addressParts[b];
            if (s.match(/^\d{4,4}$/)) {
                postcode = s;
            } else if(stateFromString(s)) {
                state = s;
            } else {
                suburbParts.push(s);
            }
        }

        suburb = suburbParts.join(' ');
        if (suburb && isString(suburb)) {
            obj["suburbOrPlaceOrLocality"] = suburb.toUpperCase();
        } else {
            throw 'invalid suburb: ' + address;
        }

        if (state && isString(state)) {
            obj["stateOrTerritory"] = state.toUpperCase();
        } else {
            throw 'invalid state: ' + address;
        }

        if (postcode) {
            obj["postcode"] = postcode;
        } else {
            throw 'invalid postcode: ' + address;
        }

        return obj;
    }

    function stateFromString(str) {
        let state = str.toUpperCase();
        let states = ['NSW','QLD','VIC','TAS','NT','ACT','SA','WA'];
        let index = _.indexOf(states, state);
        if(index == -1) {
            return null;
        } else {
            return states[index];
        }

    }

    function stateFromPostcode(postcode) {
        let pc = parseInt(postcode);

        if(pc >= 1000 && pc <= 1999)
            return 'NSW';
        if(pc >= 2000 && pc <= 2599)
            return 'NSW';
        if(pc >= 2620 && pc <= 2899)
            return 'NSW';
        if(pc >= 2921 && pc <= 2999)
            return 'NSW';
        
        if(pc >= 200 && pc <= 299)
            return 'ACT';
        if(pc >= 2600 && pc <= 2619)
            return 'ACT';
        if(pc >= 2900 && pc <= 2920)
            return 'ACT';
        
        if(pc >= 3000 && pc <= 3999)
            return 'VIC';
        if(pc >= 8000 && pc <= 8999)
            return 'VIC';
        
        if(pc >= 4000 && pc <= 4999)
            return 'QLD';
        if(pc >= 9000 && pc <= 9999)
            return 'QLD';
        
        if(pc >= 5000 && pc <= 5799)
            return 'SA';
        if(pc >= 5800 && pc <= 5999)
            return 'SA';
        
        if(pc >= 6000 && pc <= 6797)
            return 'WA';
        if(pc >= 6800 && pc <= 6999)
            return 'WA';
        
        if(pc >= 7000 && pc <= 7799)
            return 'TAS';
        if(pc >= 7800 && pc <= 7999)
            return 'TAS';
        
        if(pc >= 800 && pc <= 899)
            return 'NT';
        if(pc >= 900 && pc <= 999)
            return 'NT';
      
      return "UNKNOWN";
    }

    function poBoxType(poBoxType) {
        poBoxType = poBoxType.trim().toUpperCase();
        poBoxType = poBoxType.replace(/[\t\s]+/g, ' ');

        return poBoxType;
    }

    
    return _helper;
    
})();
    
module.exports = _helper;