"use strict";

let AWS = require('aws-sdk');

let creds;
if (process.env.hasOwnProperty('AWS_PROFILE')) {
    creds = new AWS.SharedIniFileCredentials({profile: process.env.AWS_PROFILE});
} else {
    creds = new AWS.EnvironmentCredentials('AWS'); // Lambda provided credentials
}

const config = {
    credentials: creds,
    region: 'ap-southeast-2' //process.env.AWS_REGION
};
const kms = new AWS.KMS(config);

let crypt = (function() {

    let crypt = function() {};

    crypt.prototype.encryptObject = function(object, kmsKeyId, cb) {
        encrypt(object, kmsKeyId, cb);
    };
    
    crypt.prototype.decryptObject = function(ssnText, cb) {
        decrypt(ssnText, cb);
    };
   
    let decrypt = function(ssnText,  cb) {
        var params = {
            CiphertextBlob: Buffer(ssnText, 'base64')
        };
        
        kms.decrypt(params, function(err, data) {
            if (err){
                return cb(err, null); // an error occurred
            } 
            return cb(null, data.Plaintext.toString());   // successful response
        });
    };

    let encrypt = function(object, kmsKeyId, cb) {
        var params = {
            KeyId: kmsKeyId, /* required */
            Plaintext: object
        };

        kms.encrypt(params, function(err, data) {
            if (err){
                return cb(err, null); // an error occurred
            } 
            return cb(null, data.CiphertextBlob);   // successful response
        });
    };

    return crypt;

})();

module.exports = crypt;
