'use strict';

const Contact = require('./lib/hubspot/contact')
const Deal = require('./lib/hubspot/deal')
const Engagement = require('./lib/hubspot/engagement')
const Company = require('./lib/hubspot/company');
const CRM = require('./lib/hubspot/crm');

const request = require('request-promise')
const _ = require('lodash');

// define how long to wait API response before throwing a timeout error
const API_TIMEOUT = 15000

class HubspotHelper {
    
    constructor(options = {}) {
      this.qs = {}
      this.auth = undefined
      this.setAuth(options)
      
      this.baseUrl = options.baseUrl || 'https://api.hubapi.com'
      this.apiTimeout = options.timeout || API_TIMEOUT
      
      this.contacts = new Contact(this)
      this.companies = new Company(this)
      this.deals = new Deal(this)
      this.engagements = new Engagement(this)
      this.crm = new CRM(this)

      this.callerId = options.callerId || process.env.AWS_LAMBDA_FUNCTION_NAME || process.env.npm_package_name || 'Unknown';
        
      }

      setAuth(options = {}) {
        if (options.apiKey) {
          this.qs.hapikey = options.apiKey
        }  
      }
    
      _request(opts) {
        console.log('_request execution began with following options: ', opts)  
        const params = _.cloneDeep(opts)
        if (this.auth) {
          params.auth = this.auth
        }
        params.json = true
        params.resolveWithFullResponse = true
    
        params.url = this.baseUrl + params.path
        delete params.path
        params.qs = Object.assign({}, this.qs, params.qs)
    
        params.qsStringifyOptions = {
          arrayFormat: 'repeat',
        }
    
        params.timeout = this.apiTimeout

        params.headers = Object.assign({'CallerID': this.callerId}, params.headers);

        console.log('Initiating request with following parameters: ', params)

        return request(params).then(res => {
            console.log('Response: ', res)
            return res
          })
          .then(res => res.body)
          
      }
}

module.exports = HubspotHelper;