var request = require('request');
var _ = require('lodash');

let _helper = (function() {

    let configurationManager;
    let config;
    let apiVersion = 'v1';

    let _helper = function(_configurationManager) {
        configurationManager = _configurationManager;
    };

    _helper.prototype.createPaymentSchedule = function(obj, cb) {
        let path = 'paymentSchedulePeriods';

        let requestObj = {
            "account.id": obj.accountId,
            fromDt: obj.fromDate,
            // toDt: obj.toDate,
            frequency: obj.frequency,
            frequencyMultiple: obj.frequencyMultiple,
            frequencyAlignmentDt: obj.frequencyAlignmentDt,
            seasonalPaymentFl: false,
            amount: obj.amount
        };

        makeCall(path, null, requestObj, cb);
    };

    _helper.prototype.searchAccounts = function(obj, cb) {
        let path = 'accounts';

        let requestObj = {
            surname: obj.surname
        };

        makeCall(path, requestObj, null, cb);
    };

    _helper.prototype.getAccountId = function(obj, cb) {
        let path = 'accounts';

        let requestObj = {
            number: obj.number
        };

        makeCall(path, requestObj, null, cb);
    };

    _helper.prototype.getAccountBills = function(obj, cb) {
        let path = `accounts/${obj.id}/bills`;

        let requestObj = {
            
        };

        makeCall(path, requestObj, null, cb);
    };

    function makeEndPoint(path) {
        return new Promise((resolve, reject) => {
            if(!_.isEmpty(config)) { // use the cached version
                resolve(`https://${config['hostname']}:${config['port']}/rest/${apiVersion}/${path}`);
            } else {
                getConfig()
                .then((configReturned) => {
                    config = configReturned;    // store for later
                    console.log(JSON.stringify(configReturned));
                    resolve(`https://${config['hostname']}:${config['port']}/rest/${apiVersion}/${path}`);
                })
                .catch((err) => {
                    reject(err);
                });
            }
        });
    }

    function getConfig() {
        return new Promise((resolve, reject) => {
            configurationManager.getCredentials('API', 'JUNIFER-PROD', function(err, credentials){
                if(err){ 
                    reject(err);
                } else {
                    resolve(credentials);
                }
                
            });
        });
    }

    function makeCall(path, getParams, postBody, cb) {
        makeEndPoint(path)
        .then((endpoint) => {
            // setup common options
            var options = {
                url: endpoint, 
                json: true,
                timeout: 15000,
                time: true
            };
        
            // setup method and any body / qs params
            if(postBody) {
                options['body'] = postBody;
                options['method'] = 'POST';
            } else {
                options['method'] = 'GET';
                if(getParams) {
                    options['qs'] = getParams;
                }
            }
        
            // add authentication tokens
            options['headers'] = {
                'X-Junifer-X-username': config['user'],
                'X-Junifer-X-apikey': config['apikey']
            };
        
            //console.log(JSON.stringify(options));
        
            request(options, function(error, response, body){
                if(error) {
                    cb(error, null);
                } else {
                    let result = {
                        statusCode: response.statusCode,
                        body,
                        timings: response.timingPhases
                    };
                    cb(null, result);
                }
            });
        })
        .catch((err) => {
            console.log(err);
            console.log(JSON.stringify(options));
            return cb('error', null);
        });
         
        
    }
    
      
    return _helper;
    
})();
    
module.exports = _helper;
