var redis = require('redis');

let _helper = (function() {

    let _client;
    
    let _helper = function() {};

    _helper.prototype.enter = function(callback) {
        try {
            _client = redis.createClient(
                {
                    port: '6379',
                    host: 'redis-lambdajobs-001.x3utju.0001.apse2.cache.amazonaws.com',
                    //host: 'localhost',
                    retry_strategy: retryStrategy,
                    connect_timeout: 1000
                }
            );
            
            _client.on('connect', function(){
                console.log('entered...');
                callback(null, true);
            });
            
            _client.on('error', function(err){
                callback('error1: ' + JSON.stringify(err), null);
            });
        } catch (ex) {
            callback('error2: ' + JSON.stringify(ex), null);
        }
    };

    _helper.prototype.end = function(callback) {
        _client.quit();
        _client.on('end',function(){
            callback(null, true);
        });
    };

    _helper.prototype.getValue = function(key, callback) {
        console.log('getting...' + key);
        
        _client.get(key, function (err, cachedData) {
            if(err)
            {
                callback('error!: ' + JSON.stringify(err), null);
            }else{
                if(cachedData === null || typeof cachedData === "undefined" || cachedData === undefined || cachedData == 'null'){
                    callback(null, 'KEY MISS');
                }else{
                    callback(null, cachedData);
                }
            }
        });
    };

    _helper.prototype.setValue = function(key, value, lifetime, callback) {
        console.log('setting...' + key);
        _client.set(key, value, function (err, data) {
            if(err)
            {
                callback('error!: ' + JSON.stringify(err), null);
            }else{
                _client.expire(key, lifetime);
                callback(null, value);
            }
        });
    };

    _helper.prototype.incrementValue = function(key, callback) {
        console.log('incrementing...' + key);
        _client.incr(key, function (err, data) {
            if(err)
            {
                callback('error!: ' + JSON.stringify(err), null);
            }else{
                _client.expire(key, 3600);
                console.log('counter: ' + data + ' key:' + key);
                callback(null, data);
            }
        });
    };

    _helper.prototype.deleteKey = function(key, callback) {
        console.log('deleting...' + key);
        _client.del(key, function (err, data) {
            if(err)
            {
                callback('error!: ' + JSON.stringify(err), null);
            }else{
                callback(null, data);
            }
        });
    };


    _helper.prototype.addMemberToSet = function(set, id, callback) {
        //console.log('adding id ' + id + ' to set ' + set);
        

            _client.sadd(set, id, function (err, data) {
                if(err)
                {
                    callback('error!: ' + JSON.stringify(err), null);
                }else{
                    callback(null, data);
                }
            });

    }

    _helper.prototype.isMemberOfSet = function(set, id, callback) {
        console.log('checking if id ' + id + ' is a member of set ' + set);
        
        _client.sismember(set, id, function (err, data) {
            if(err)
            {
                callback('error!: ' + JSON.stringify(err), null);
            }else{
                callback(null, data);
            }
        });
        
    }

    let retryStrategy = function(options) {
        if (options.error && options.error.code === 'ECONNREFUSED') {
            // End reconnecting on a specific error and flush all commands with a individual error
            return new Error('The server refused the connection');
        }
        if (options.total_retry_time > 1000 ) {
            // End reconnecting after a specific timeout and flush all commands with a individual error
            return new Error('Retry time exhausted');
        }
        if (options.attempt > 1) {
            // End reconnecting with built in error
            return undefined;
        }
        // reconnect after
        return Math.min(options.attempt * 100, 3000);
    };

    return _helper;
    
})();
    
module.exports = _helper;
