'use strict';

let AWS = require('aws-sdk');
let S3Bucket = require('./s3_bucket.js');

class S3Helper {
    /**
     * @constructor
     */
    constructor(params) {
        let creds = null;
        if (typeof(params) == 'object' && params.hasOwnProperty('profile')) {
            creds = new AWS.SharedIniFileCredentials({profile: params.profile});
        } else if (process.env.hasOwnProperty('AWS_PROFILE')) {
            creds = new AWS.SharedIniFileCredentials({profile: process.env.aws_profile});
        } else {
            creds = new AWS.EnvironmentCredentials('AWS');
        }

        this._awsConfig = {
            credentials: creds,
            region: 'ap-southeast-2'
        };

        if (typeof(params) == 'object' && params.hasOwnProperty('signatureVersion')) {
            this._awsConfig['signatureVersion'] = params.signatureVersion;
        }

        this._s3 = null;
        this._buckets = {}
    }

    /**
     * create AWS.S3 object
     * 
     * @public
     * @returns {object} AWS.S3 object
     */
    getS3() {
        if (!this._s3) {
            this._s3 = new AWS.S3(this._awsConfig);
        }

        return this._s3;
    }

    /**
     * create a S3Bucket object and return it
     * 
     * @public
     * @param {string} bucketName name of the bucket that you want to work with
     * @returns {object} S3Bucket object
     */
    getBucket(bucketName) {
        let bucket = null;

        if (this._buckets.hasOwnProperty(bucketName)) {
            bucket = this._buckets[bucketName];
        } else {
            bucket = new S3Bucket(bucketName, this);
        }

        return bucket;
    }
}

module.exports = S3Helper;
