'use strict';

var jsforce = require('jsforce');

class SalesforceHelper {
    /**
     * @constructor
     * @param {object} _configurationManager
     */
    constructor(_configurationManager) {
        this._configurationManager = _configurationManager;
        this._conn = null;
    }

    /**
     * Connect and login into Salesforce
     * 
     * @param {string} instanceOf 
     */
    connect(instanceOf) {
        return new Promise((resolve, reject) => {
            this._configurationManager.getCredentials('API', instanceOf, (err, credentials) => {
                if(err){ 
                    reject(err);
                } else {
                    this._conn = new jsforce.Connection({
                        oauth2 : {
                            loginUrl : credentials.login_url,
                            clientId : credentials.client_id,
                            clientSecret : credentials.clientSecret,
                            redirectUri : '<callback URI is here>'
                        }
                    });

                    this._conn.login(credentials.username, credentials.password + credentials.token, (err, userInfo) => {
                        if (err) {
                            reject(err);
                        } else {
                            resolve('CONNECTED');
                        }
                    });
                }
            });
        });
    }

    /**
     * Query to Salesforce
     * 
     * @param {string} soql 
     */
    query(soql) {
        return new Promise((resolve, reject) => {
            this._conn.query(soql, (err, res) => {
                if (err) {
                    reject(err);
                } else {
                    resolve(res);
                }
            });
        });
    }

    /**
     * Create an object in Salesforce
     * 
     * @param {string} objectName 
     * @param {object} payload 
     */
    create(objectName, payload) {
        return new Promise((resolve, reject) => {
            this._conn.sobject(objectName).create(payload, (err, res) => {
                if (err) {
                    reject(err);
                } else {
                    resolve(res);
                }
            });
        });
    }
}

module.exports = SalesforceHelper;