'use strict';

let _ = require('lodash');

class SQSQueue {
    
    /**
     * @constructor
     * @param {string} queueName queue name
     * @param {object} helper SQSHelper object
     */
    constructor(queueName, helper) {
        this._queueName = queueName;
        this._helper = helper;
        
        this._queueUrl = null;
    }

    getQueueUrl(cb) {
        if (this._queueUrl) {
            cb(null, this._queueUrl);
        } else {
            let params = {
                QueueName: this._queueName
            };
            this._helper.getSQS().getQueueUrl(params, (err, data) => {
                if (err) {
                    cb(err, null);
                } else {
                    this._queueUrl = data.QueueUrl;
                    cb(null, this._queueUrl);
                }
            });
        }
    }

    sendMessage(obj, extraParams, cb) {
        this.getQueueUrl((err, url) => {
            if (err) {
                cb(err, null);
            } else {
                let params = {
                    QueueUrl: url,
                    MessageBody: JSON.stringify(obj)
                };
                if (extraParams) {
                    params = _.assign(params, extraParams);
                }
                this._helper.getSQS().sendMessage(params, cb);
            }
        });
    }

    receiveMessage(num, extraParams, cb) {
        this.getQueueUrl((err, url) => {
            if (err) {
                cb(err, null);
            } else {
                let params = {
                    QueueUrl: url,
                    MaxNumberOfMessages: num
                };
                if (extraParams) {
                    params = _.assign(params, extraParams);
                }
                this._helper.getSQS().receiveMessage(params, cb);
            }
        });
    }

    deleteMessage(receiptHandle, extraParams, cb) {
        this.getQueueUrl((err, url) => {
            if (err) {
                cb(err, null);
            } else {
                let params = {
                    QueueUrl: url,
                    ReceiptHandle: receiptHandle
                };
                if (extraParams) {
                    params = _.assign(params, extraParams);
                }
                this._helper.getSQS().deleteMessage(params, cb);
            }
        });
    }
}

module.exports = SQSQueue;
