let _ = require('lodash');
let util = require('util');
let fs = require('fs');
let path = require('path');
let crypto = require('crypto');

// uuid generator
function uuid(a){return a?(a^Math.random()*16>>a/4).toString(16):([1e7]+-1e3+-4e3+-8e3+-1e11).replace(/[018]/g,uuid)}
// token generator
function tokenGenerator(l){return crypto.randomBytes(l).toString('hex');};

function formatAsMoney(str)
{
	var number = parseFloat(str);
	var money = number.toFixed(2).replace(/(\d)(?=(\d{3})+\.)/g, '$1,');
	return money;
}

function properCase(str) {
    return _.startCase(_.toLower(str));
}

function extractJson(message) {
    var jsonStart = message.indexOf('{');
    if (jsonStart < 0) return null;
    var jsonSubString = message.substring(jsonStart);
    return isValidJson(jsonSubString) ? jsonSubString : null;
}

function isValidJson(message) {
    try {
        JSON.parse(message);
    } catch (e) { return false; }
    return true;
}


function parseJsonSafely(message){
	let jsonString = extractJson(message);
	if(_.isNull(jsonString) === false) {
		return JSON.parse(jsonString);
	} else {
		return {};
	}
}

function postcodeToState(postcode) {
    postcode = parseInt(postcode);
    let state;
    if (postcode >= 1000 && postcode < 2600) {
        state = 'NSW';
    } else if (postcode >= 2619 && postcode < 2900) {
        state = 'NSW';
    } else if (postcode >= 2921 && postcode < 3000) {
        state = 'NSW';
    } else if (postcode >= 200 && postcode < 300) {
        state = 'ACT';
    } else if (postcode >= 2600 && postcode < 2619) {
        state = 'ACT';
    } else if (postcode >= 2900 && postcode < 2921) {
        state = 'ACT';
    } else if (postcode >= 3000 && postcode < 4000) {
        state = 'VIC';
    } else if (postcode >= 8000 && postcode < 9000) {
        state = 'VIC';
    } else if (postcode >= 4000 && postcode < 5000) {
        state = 'QLD';
    } else if (postcode >= 9000 && postcode < 10000) {
        state = 'QLD';
    } else if (postcode >= 5000 && postcode < 6000) {
        state = 'SA';
    } else if (postcode >= 6000 && postcode < 7000) {
        state = 'WA';
    } else if (postcode >= 7000 && postcode < 8000) {
        state = 'TAS';
    } else if (postcode >= 800 && postcode < 1000) {
        state = 'NT';
    } else {
        state = null;
    }
    return state;
}

/**
 * Print out the given object 
 * 
 * @function inspect
 * @param {*} obj 
 * @param {string=} label
 */
let inspect;
if (process.env.hasOwnProperty('AWS_LAMBDA_FUNCTION_NAME')) {
	inspect = function(obj) {
		if (arguments.length > 1) {
			console.log(arguments[1] + ":" + util.inspect(obj, false, null));
		} else {
			console.log(util.inspect(obj, false, null));
		}
	}
} else {
	inspect = require('eyes').inspector({
		maxLength: false, 
		styles: {
			all:     'cyan',      // Overall style applied to everything
			label:   'underline', // Inspection labels, like 'array' in `array: [1, 2, 3]`
			other:   'inverted',  // Objects which don't have a literal representation, such as functions
			key:     'bold',      // The keys in object literals, like 'a' in `{a: 1}`
			special: 'grey',      // null, undefined...
			string:  'green',
			number:  'red',       // Windows PowerShell cannot display magenta color :(
			bool:    'red',      // true false
			regexp:  'green'
		}
	});
}

function test(handler, argv, callback) {
	let err;
    let _event;
    let _context = {
        timeLimit: 30000,
        startTime: Date.now(),
        getRemainingTimeInMillis: function() {
            let currTime = Date.now();
            let difference = currTime - this.startTime;
            console.log(this.startTime + ' - ' + currTime + ' = ' + difference);
            let remainingTime = this.timeLimit - difference;
            return remainingTime;
        }
    };

    if (argv.hasOwnProperty('payload')) {
		let samplePayloadFile = path.resolve('./test/sample-payload.js');

        if (fs.existsSync(samplePayloadFile)) {
            let samplePayload = require(samplePayloadFile);
            if (samplePayload.hasOwnProperty(argv.payload)) {
                _event = samplePayload[argv.payload];
            } else {
                err = `The specified payload '${argv.payload}' is not available in the sample payload file`;
            }
        } else {
            err = `The sample payload does not exist`;
        }
    }
    
    if (err) {
        callback(err, null);
    } else {
        handler(_event, _context, callback);
    }
}

function getRetailerNameById(id) {
    let name;
    id = parseInt(id);
    switch (id) {
        case 1:
            name = 'Mojo Power';
            break;
        case 2:
            name = 'Sanctuary Energy';
            break;
        case 3:
            name = 'People Energy';
            break;
        case 4:
            name = 'QEnergy';
            break;
        default:
            name = null;
            break;
    }
    return name;
}

function getRetailerIdByName(name) {
    let id;
    name = "" + name;
    name = name.replace(/[\s_-]*/g,'').toLowerCase();
    switch (name) {
        case 'mojopower':
            id = 1;
            break;
        case 'sanctuaryenergy':
            id = 2;
            break;
        case 'peopleenergy':
            id = 3;
            break;
        case 'qenergy':
            id = 4;
            break;
        default:
            id = null;
            break;
    }
    return id;
}

function retailerNameToJuniferName(name) {
    let juniferName;
    name = "" + name;
    name = name.replace(/[\s_-]*/g,'').toLowerCase();
    switch (name) {
        case 'mojopower':
            juniferName = 'Junifer';
            break;
        case 'sanctuaryenergy':
            juniferName = 'Junifer';
            break;
        case 'peopleenergy':
            juniferName = 'JuniferPE';
            break;
        case 'qenergy':
            juniferName = 'JuniferQE';
            break;
        default:
            juniferName = null;
            break;
    }
    return juniferName;
}

function retailerIdToJuniferName(id) {
    let juniferName;
    id = parseInt(id);
    switch (id) {
        case 1:
            juniferName = 'Junifer';
            break;
        case 2:
            juniferName = 'Junifer';
            break;
        case 3:
            juniferName = 'JuniferPE';
            break;
        case 4:
            juniferName = 'JuniferQE';
            break;
        default:
            juniferName = null;
            break;
    }
    return juniferName;
}

function retailerNameToJuniferProfile(name) {
    let juniferName;
    name = "" + name;
    name = name.replace(/[\s_-]*/g,'').toLowerCase();
    switch (name) {
        case 'mojopower':
            juniferName = 'JUNIFER';
            break;
        case 'sanctuaryenergy':
            juniferName = 'JUNIFER';
            break;
        case 'peopleenergy':
            juniferName = 'JUNIFER-PE';
            break;
        case 'qenergy':
            juniferName = 'JUNIFER-QE';
            break;
        default:
            juniferName = null;
            break;
    }
    return juniferName;
}

function retailerIdToJuniferProfile(id) {
    let juniferName;
    id = parseInt(id);
    switch (id) {
        case 1:
            juniferName = 'JUNIFER';
            break;
        case 2:
            juniferName = 'JUNIFER';
            break;
        case 3:
            juniferName = 'JUNIFER-PE';
            break;
        case 4:
            juniferName = 'JUNIFER-QE';
            break;
        default:
            juniferName = null;
            break;
    }
    return juniferName;
}

module.exports = {
	uuidGenerator: uuid,
	formatAsMoney,
	convertToProperCase: properCase,
    parseJsonSafely,
    postcodeToState,
	inspect: inspect,
    test,
    tokenGenerator,
    getRetailerNameById,
    getRetailerIdByName,
    retailerNameToJuniferName,
    retailerIdToJuniferName,
    retailerNameToJuniferProfile,
    retailerIdToJuniferProfile
}
